@@ -12,20 +12,6 @@
 -- > type Name = H.Name ()
 -- > pattern Ident a = H.Ident () a
 -- > pattern Symbol a = H.Symbol () a
---
--- This works nicely for all datatypes with two exception:
---
--- * `ImportDecl` has a record constructor. Record type synonyms are only
---   supported ghc-8.0 and later, so for ghc-7.10 and earlier, the
---   constructor is exported as a plain constructor, and the record fields
---   as function.
--- * `Literal` has constructors with an extra `String` argument that is not
---   used by `Language.Haskell.Exts.Simple.Pretty`. This module uses explicitly
---   bidirectional pattern synonyms to support this type, but support for that
---   is only available in ghc-7.10 and later.
---
--- __IMPORTANT__: if you require compatiblity with ghc 7.8, you should use the
--- functions `charL`, `stringL` etc. for constructing `Literal` values!
 
 module Language.Haskell.Exts.Simple.Syntax (
     module Language.Haskell.Exts.Simple.Syntax,
@@ -119,27 +105,8 @@ pattern PatternNamespace = H.PatternNamespace () :: Namespace
 
 -- ** `H.ImportDecl`
 type ImportDecl = H.ImportDecl ()
--- | Note, this is originally a record constructor, and we use a pattern record synonym for ghc-8.0. But for earlier ghc versions, `ImportDecl` is a plain pattern synonym, and the selectors are exported as functions.
-#if __GLASGOW_HASKELL__ < 800
-pattern ImportDecl a b c d e f g = H.ImportDecl () (a :: ModuleName) (b :: Bool) (c :: Bool) (d :: Bool) (e :: Maybe String) (f :: Maybe ModuleName) (g :: Maybe ImportSpecList) :: ImportDecl
-importModule :: ImportDecl -> ModuleName
-importModule = H.importModule
-importQualified :: ImportDecl -> Bool
-importQualified = H.importQualified
-importSrc :: ImportDecl -> Bool
-importSrc = H.importSrc
-importSafe :: ImportDecl -> Bool
-importSafe = H.importSafe
-importPkg :: ImportDecl -> Maybe String
-importPkg = H.importPkg
-importAs :: ImportDecl -> Maybe ModuleName
-importAs = H.importAs
-importSpecs :: ImportDecl -> Maybe ImportSpecList
-importSpecs = H.importSpecs
-#else
 pattern ImportDecl { importModule, importQualified, importSrc, importSafe, importPkg, importAs, importSpecs } =
     H.ImportDecl () importModule importQualified importSrc importSafe importPkg importAs importSpecs :: ImportDecl
-#endif
 
 -- ** `H.ImportSpecList`
 type ImportSpecList = H.ImportSpecList ()
@@ -175,8 +142,9 @@ pattern DerivDecl a b c = H.DerivDecl () (a :: (Maybe DerivStrategy)) (b :: (May
 pattern InfixDecl a b c = H.InfixDecl () (a :: Assoc) (b :: (Maybe Int)) (c :: [Op]) :: Decl
 pattern DefaultDecl a = H.DefaultDecl () (a :: [Type]) :: Decl
 pattern SpliceDecl a = H.SpliceDecl () (a :: Exp) :: Decl
+pattern TSpliceDecl a = H.TSpliceDecl () (a :: Exp) :: Decl
 pattern TypeSig a b = H.TypeSig () (a :: [Name]) (b :: Type) :: Decl
-pattern PatSynSig a b c d e = H.PatSynSig () (a :: [Name]) (b :: (Maybe [TyVarBind])) (c :: (Maybe Context)) (d :: (Maybe Context)) (e :: Type) :: Decl
+pattern PatSynSig a b c d e f = H.PatSynSig () (a :: [Name]) (b :: (Maybe [TyVarBind])) (c :: (Maybe Context)) (d :: (Maybe [TyVarBind])) (e :: (Maybe Context)) (f :: Type) :: Decl
 pattern FunBind a = H.FunBind () (a :: [Match]) :: Decl
 pattern PatBind a b c = H.PatBind () (a :: Pat) (b :: Rhs) (c :: (Maybe Binds)) :: Decl
 pattern PatSyn a b c = H.PatSyn () (a :: Pat) (b :: Pat) (c :: PatternSynDirection) :: Decl
@@ -267,6 +235,7 @@ type DerivStrategy = H.DerivStrategy ()
 pattern DerivStock = H.DerivStock () :: DerivStrategy
 pattern DerivAnyclass = H.DerivAnyclass () :: DerivStrategy
 pattern DerivNewtype = H.DerivNewtype () :: DerivStrategy
+pattern DerivVia a = H.DerivVia () (a :: Type) :: DerivStrategy
 
 -- ** `H.Binds`
 type Binds = H.Binds ()
@@ -298,7 +267,7 @@ pattern FieldDecl a b = H.FieldDecl () (a :: [Name]) (b :: Type) :: FieldDecl
 
 -- ** `H.GadtDecl`
 type GadtDecl = H.GadtDecl ()
-pattern GadtDecl a b c = H.GadtDecl () (a :: Name) (b :: (Maybe [FieldDecl])) (c :: Type) :: GadtDecl
+pattern GadtDecl a b c d e = H.GadtDecl () (a :: Name) (b :: (Maybe [TyVarBind])) (c :: (Maybe Context)) (d :: (Maybe [FieldDecl])) (e :: Type) :: GadtDecl
 
 -- ** `H.ClassDecl`
 type ClassDecl = H.ClassDecl ()
@@ -339,6 +308,7 @@ pattern GuardedRhs a b = H.GuardedRhs () (a :: [Stmt]) (b :: Exp) :: GuardedRhs
 -- ** `H.Type`
 type Type = H.Type ()
 pattern TyForall a b c = H.TyForall () (a :: (Maybe [TyVarBind])) (b :: (Maybe Context)) (c :: Type) :: Type
+pattern TyStar = H.TyStar () :: Type
 pattern TyFun a b = H.TyFun () (a :: Type) (b :: Type) :: Type
 pattern TyTuple a b = H.TyTuple () (a :: Boxed) (b :: [Type]) :: Type
 pattern TyUnboxedSum a = H.TyUnboxedSum () (a :: [Type]) :: Type
@@ -379,14 +349,9 @@ pattern KindedVar a b = H.KindedVar () (a :: Name) (b :: Kind) :: TyVarBind
 pattern UnkindedVar a = H.UnkindedVar () (a :: Name) :: TyVarBind
 
 -- ** `H.Kind`
+
+-- | Note that `Kind` is an alias for `Type` since haskell-src-exts-1.21.
 type Kind = H.Kind ()
-pattern KindStar = H.KindStar () :: Kind
-pattern KindFn a b = H.KindFn () (a :: Kind) (b :: Kind) :: Kind
-pattern KindParen a = H.KindParen () (a :: Kind) :: Kind
-pattern KindVar a = H.KindVar () (a :: QName) :: Kind
-pattern KindApp a b = H.KindApp () (a :: Kind) (b :: Kind) :: Kind
-pattern KindTuple a = H.KindTuple () (a :: [Kind]) :: Kind
-pattern KindList a = H.KindList () (a :: Kind) :: Kind
 
 -- ** `H.FunDep`
 type FunDep = H.FunDep ()
@@ -400,29 +365,13 @@ pattern CxEmpty = H.CxEmpty () :: Context
 
 -- ** `H.Asst`
 type Asst = H.Asst ()
-pattern ClassA a b = H.ClassA () (a :: QName) (b :: [Type]) :: Asst
-pattern AppA a b = H.AppA () (a :: Name) (b :: [Type]) :: Asst
-pattern InfixA a b c = H.InfixA () (a :: Type) (b :: QName) (c :: Type) :: Asst
+pattern TypeA a = H.TypeA () (a :: Type) :: Asst
 pattern IParam a b = H.IParam () (a :: IPName) (b :: Type) :: Asst
-pattern EqualP a b = H.EqualP () (a :: Type) (b :: Type) :: Asst
 pattern ParenA a = H.ParenA () (a :: Asst) :: Asst
-pattern WildCardA a = H.WildCardA () (a :: (Maybe Name)) :: Asst
 
 -- ** `H.Literal`
-
--- | Beware that the constructors only work in a pattern context in ghc-7.8,
--- because that version does not support explicitly bidirectional pattern
--- synonyms.
---
--- For code that needs to work with ghc-7.8, we provide functions `charL`,
--- `stringL`, `intL`, `fracL`, etc. for constructing `Literal` values.
-
 type Literal = H.Literal ()
 
-#if __GLASGOW_HASKELL__ <= 708
-#define where --
-#endif
-
 pattern Char a <- H.Char () (a :: Char) _ :: Literal
     where Char a = H.Char () a [a]
 charL :: Char -> Literal
@@ -550,6 +499,7 @@ pattern XAttr a b = H.XAttr () (a :: XName) (b :: Exp) :: XAttr
 -- ** `H.Bracket`
 type Bracket = H.Bracket ()
 pattern ExpBracket a = H.ExpBracket () (a :: Exp) :: Bracket
+pattern TExpBracket a = H.TExpBracket () (a :: Exp) :: Bracket
 pattern PatBracket a = H.PatBracket () (a :: Pat) :: Bracket
 pattern TypeBracket a = H.TypeBracket () (a :: Type) :: Bracket
 pattern DeclBracket a = H.DeclBracket () (a :: [Decl]) :: Bracket
@@ -557,7 +507,9 @@ pattern DeclBracket a = H.DeclBracket () (a :: [Decl]) :: Bracket
 -- ** `H.Splice`
 type Splice = H.Splice ()
 pattern IdSplice a = H.IdSplice () (a :: String) :: Splice
+pattern TIdSplice a = H.TIdSplice () (a :: String) :: Splice
 pattern ParenSplice a = H.ParenSplice () (a :: Exp) :: Splice
+pattern TParenSplice a = H.TParenSplice () (a :: Exp) :: Splice
 
 -- ** `H.Safety`
 type Safety = H.Safety ()
@@ -710,6 +662,9 @@ unit_con_name = H.unit_con_name ()
 tuple_con_name :: Boxed -> Int -> QName
 tuple_con_name = H.tuple_con_name ()
 
+list_con_name :: QName
+list_con_name = H.list_con_name ()
+
 list_cons_name :: QName
 list_cons_name = H.list_cons_name ()
 
@@ -737,7 +692,7 @@ star_name = H.star_name ()
 hole_name :: QName
 hole_name = H.hole_name ()
 
-export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name, stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name, javascript_name, capi_name, forall_name, family_name, role_name, stock_name, anyclass_name :: Name
+export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name, stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name, javascript_name, capi_name, forall_name, family_name, role_name, stock_name, anyclass_name, via_name :: Name
 export_name = H.export_name ()
 safe_name = H.safe_name ()
 unsafe_name = H.unsafe_name ()
@@ -756,6 +711,7 @@ family_name = H.family_name ()
 role_name = H.role_name ()
 stock_name = H.stock_name ()
 anyclass_name = H.anyclass_name ()
+via_name = H.via_name ()
 
 unit_tycon_name, fun_tycon_name, list_tycon_name, unboxed_singleton_tycon_name :: QName
 unit_tycon_name = H.unit_tycon_name ()
